#!/usr/bin/env python3
# coding=utf-8
import sys
import re
import os
import base64

RE_REMOTEIMG = re.compile('^(http|https):.+')

_meta_data_fence_pattern = re.compile(r'^---[\ \t]*\n', re.MULTILINE)
_meta_data_pattern = re.compile(
    r'^(?:---[\ \t]*\n)?(.*:\s+>\n\s+[\S\s]+?)(?=\n\w+\s*:\s*\w+\n|\Z)|([\S\w]+\s*:(?! >)[ \t]*.*\n?)(?:---[\ \t]*\n)?',
    re.MULTILINE)


def unique_list(alist):
    return list(set(alist))


def clean_list(alist):
    """
    clean items
    :param list alist: a list
    :return:
    """
    alist = list(map(lambda x: str(x).strip(), alist))
    alist = list(filter(lambda x: x != "", alist))
    return unique_list(alist)


def get_first(alist):
    rlist = clean_list(alist) or [None]
    return rlist[0]


def to_bool(value):
    if isinstance(value, (list, tuple)):
        value = value[0]
    value = str(value)
    if value.strip() in [0, None, "None", "False", "", "0"]:
        return False
    return True


# Simplified for Python 3 only
def utf8(value):
    """Converts a string argument to a byte string.

    If the argument is already a byte string or None, it is returned unchanged.
    Otherwise it must be a unicode string and is encoded as utf8.
    """
    if isinstance(value, (bytes, type(None))):
        return value
    if not isinstance(value, str):
        raise TypeError(
            "Expected bytes, unicode, or None; got %r" % type(value)
        )
    return value.encode("utf-8")


def to_unicode(value):
    """Converts a string argument to a unicode string.

    If the argument is already a unicode string or None, it is returned
    unchanged.  Otherwise it must be a byte string and is decoded as utf8.
    """
    if isinstance(value, (str, type(None))):
        return value
    if not isinstance(value, bytes):
        raise TypeError(
            "Expected bytes, unicode, or None; got %r" % type(value)
        )
    return value.decode("utf-8")


def recursive_unicode(obj):
    """Walks a simple data structure, converting byte strings to unicode.

    Supports lists, tuples, and dictionaries.
    """
    if isinstance(obj, dict):
        return dict((recursive_unicode(k), recursive_unicode(v)) for (k, v) in obj.items())
    elif isinstance(obj, list):
        return list(recursive_unicode(i) for i in obj)
    elif isinstance(obj, tuple):
        return tuple(recursive_unicode(i) for i in obj)
    elif isinstance(obj, bytes):
        return to_unicode(obj)
    else:
        return obj


def convert_img_to_b64(src, base_dir=None):
    if RE_REMOTEIMG.match(src):
        return src

    src = os.path.expanduser(src)
    if base_dir:
        base_dir = os.path.expanduser(base_dir)
        src = os.path.join(base_dir, src)

    if not os.path.exists(src):
        raise ValueError("file does not exists on: %s" % src)

    ext = "png"
    if os.path.splitext(src)[1] in [".jpg", "jpeg"]:
        ext = "jpeg"

    with open(src, "rb") as f:
        data = f.read()

    img_data = base64.b64encode(data).decode('utf-8')
    res = "data:image/%s;base64,%s" % (ext, img_data)
    return res


# 简化的图片处理类，不依赖于老版本的ImagePattern
class ImageCheckPattern:
    def __init__(self, base_dir, md_inst=None):
        self.__base_dir = base_dir
        self.md = md_inst

    def handleMatch(self, m, data):
        # 简化的处理，直接处理图片src
        # 这个方法在新版本中可能不会被调用，但保留以兼容性
        pass


def parse_title(mdstring):
    """
    get title
    """

    if mdstring.startswith("---"):
        fence_splits = re.split(_meta_data_fence_pattern, mdstring, maxsplit=2)
        if len(fence_splits) >= 3:
            metadata_content = fence_splits[1]
            match = re.findall(_meta_data_pattern, metadata_content)
            if match:
                mdstring = fence_splits[2]

    mdstring = mdstring.lstrip("\n").lstrip(" ")
    head_pattern1 = re.compile(r'^ *(#{1}) *([^\n\n]+?) *#* *(?:\n+|$)')
    m = re.match(head_pattern1, mdstring)
    if m:
        return m.group(2).strip()

    head_pattern2 = re.compile(r'^ *([^\n]+?) *\n *=+ *(?:\n+|$)')
    m = re.match(head_pattern2, mdstring)
    if m:
        return m.group(1).strip()

    return ""



#!/usr/bin/env python3
# coding=utf-8
"""
mdtree, convert markdown to html with TOC(table of contents) tree and Mermaid diagram support. https://github.com/ltto/mdtree
"""
import sys, os
import markdown
from mdtree.mdutils import clean_list, parse_title, to_bool, to_unicode, utf8
from mdtree.parser import gen_html, prepare_static_files, parse_static_files, adjust_ext_list

_d_static_path = os.path.join(os.path.dirname(os.path.abspath(__file__)), "static")

__all__ = ["MdTree", "convert_from_file"]


class MdTree(object):
    """
    Python markdown tree tool with Mermaid diagram support
    """

    def __init__(self, **kwargs):
        """
        :param kwargs:
        """
        self.ext_list = [
            "markdown.extensions.meta",
            "markdown.extensions.attr_list",
            "markdown.extensions.tables",
            "markdown.extensions.toc",
            "markdown.extensions.fenced_code",
            "markdown.extensions.codehilite",
            "markdown_mermaidjs",  # Add Mermaid support
        ]

        self.title = kwargs.get("title", "")
        self._html = ""

        self.js_list = kwargs.get("js", [])
        self.css_list = kwargs.get("css", [])
        self.ext_list.extend(kwargs.get("exts", []))
        self.ext_list = list(set(self.ext_list))

        self.by = "text"
        self.filepath = kwargs.get("filepath", None)
        self.base_dir = kwargs.get("base_dir", None)
        self.to64 = kwargs.get("to64", False)

    def adjust_config(self, meta):
        ext_list = meta.get("exts", [])
        css_list = meta.get("css", [])
        js_list = meta.get("js", [])

        self.ext_list = clean_list(list(set(self.ext_list + ext_list)))
        self.ext_list = adjust_ext_list(self.ext_list, ext_list)

        self.css_list = clean_list(list(set(self.css_list + css_list)))
        self.js_list = clean_list(list(set(self.js_list + js_list)))

        self.base_dir = meta.get("base_dir", [None])[0]
        to64_v_list = clean_list(meta.get("to64", [False]))[0]
        self.to64 = self.to64 or to_bool(to64_v_list)

        if self.by == "file" and not self.base_dir:
            self.base_dir = os.path.split(self.filepath)[0]

        self.title = self.title or meta.get("title", [""])[0]

    def parse_md_config(self, source):
        """
        parse exts config from markdown file and update the markdown object

        exts source: https://python-markdown.github.io/extensions/
        :return:
        """
        md1 = markdown.Markdown(extensions=["markdown.extensions.meta"])
        md1.convert(source)
        md_meta = getattr(md1, "Meta", {})

        # recreate an instance of Markdown object
        md2 = markdown.Markdown(extensions=self.ext_list)
        
        # Note: to64 image conversion is disabled for now due to API changes in newer markdown versions
        # This can be re-implemented using a custom processor if needed
        
        return md2, md_meta

    def convert(self, source):
        """
        convert markdown to html with TOC and Mermaid diagram support
        :param str source: contents of markdown file
        :return:
        """
        source = to_unicode(source)

        # parse meta、exts config
        md, md_meta = self.parse_md_config(source)
        self.adjust_config(md_meta)

        md_html = md.convert(source)
        toc = getattr(md, "toc", "")

        # prepare the basic static files
        css_base, js_base = prepare_static_files()

        # get title from init、meta、markdown source
        title = self.title or parse_title(source)

        # try to get more static files from markdown source
        css_more, js_more = parse_static_files(md_meta, self.css_list, self.js_list)

        params = {
            "title": title,
            "content": md_html,
            "css_base": css_base,
            "js_base": js_base,
            "css_more": css_more,
            "js_more": js_more,
            "toc": toc,
        }

        self._html = gen_html(params)
        # Ensure we return a string, not bytes
        if isinstance(self._html, bytes):
            self._html = self._html.decode('utf-8')
        return self._html

    def convert_file(self, spath):
        """
        convert markdown to html with TOC and Mermaid diagram support
        :param str spath: path of source file
        """
        self.by = "file"
        self.filepath = os.path.expanduser(spath)
        self.base_dir = os.path.split(self.filepath)[0]

        with open(spath, 'r', encoding='utf-8') as f:
            mdstring = f.read()

        return self.convert(mdstring)

    def save_file(self, tpath):
        """
        write to file
        :param str tpath: path of target file
        :return:
        """
        html_content = self._html
        if isinstance(html_content, bytes):
            html_content = html_content.decode('utf-8')
        
        with open(tpath, "w", encoding='utf-8') as f:
            f.write(html_content)
        return tpath


# Exported Funcs
def convert_from_file(**kwargs):
    """
    :param dict kwargs:
    :return:
    """
    source = kwargs["source"] or ""
    target = kwargs.get("target") or ""

    mdtree = MdTree(**kwargs)
    html = mdtree.convert_file(source)
    
    if target:
        mdtree.save_file(target)
    else:
        # For stdout, we need bytes
        html_bytes = utf8(html)
        sys.stdout.buffer.write(html_bytes)
    return html

def main():
    """
    Main entry point for command line interface
    """
    import argparse
    
    parser = argparse.ArgumentParser(
        description="convert markdown to html with TOC(Table of contents) tree and Mermaid diagram support"
    )
    parser.add_argument("source", default="", help="source file, markdown with optional Mermaid diagrams")
    parser.add_argument("-t", "--target", help="Write output to TARGET. Defaults to STDOUT.")
    parser.add_argument("--css", help="more css, http/s links")
    parser.add_argument("--js", help="more js, http/s links")
    parser.add_argument("--title", help="title")
    parser.add_argument("--to64", action="store_true", help="convert local image to base64? (temporarily disabled)")
    
    args = parser.parse_args()
    
    kwargs = {
        "source": args.source,
        "target": args.target,
        "title": args.title or "",
        "css": [args.css] if args.css else [],
        "js": [args.js] if args.js else [],
        "to64": args.to64,
    }
    
    try:
        convert_from_file(**kwargs)
    except Exception as e:
        print(f"Error: {e}", file=sys.stderr)
        sys.exit(1)

if __name__ == "__main__":
    main()



#!/usr/bin/env python3
# coding=utf-8
__version__ = "0.8.0"
__desc__ = "Convert markdown to html with TOC(table of contents) and Mermaid diagram support https://github.com/ltto/mdtree"
from mdtree.main import MdTree, convert_from_file



#!/usr/bin/env python3
# encoding: utf-8
from setuptools import setup, find_packages
import os

__version__ = "0.8.0"
__desc__ = "Convert markdown to html with TOC(table of contents) and Mermaid diagram support"

repo_url = "https://github.com/ltto/mdtree"

# 读取README文件作为长描述
def read_long_description():
    try:
        with open("README.md", "r", encoding="utf-8") as f:
            return f.read()
    except FileNotFoundError:
        return __desc__

setup(
    name="mdtree-mermaid",
    version=__version__,
    keywords=["markdown", "toc", "tree", "html", "mermaid", "documentation", "converter"],
    description=__desc__,
    long_description=read_long_description(),
    long_description_content_type="text/markdown",
    license="MIT",
    url=repo_url,
    project_urls={
        "Bug Reports": f"{repo_url}/issues",
        "Source": repo_url,
        "Documentation": repo_url,
    },
    author="menduo",
    author_email="shimenduo@gmail.com",
    packages=find_packages(),
    package_data={
        "mdtree": [
            "static/css/*.css",
            "static/js/*.js", 
            "static/html/*.html",
        ]
    },
    include_package_data=True,
    scripts=["bin/mdtree"],
    platforms="any",
    classifiers=[
        "Development Status :: 4 - Beta",
        "Intended Audience :: Developers",
        "Intended Audience :: End Users/Desktop",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8", 
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
        "Programming Language :: Python :: 3.11",
        "Programming Language :: Python :: 3.12",
        "Topic :: Internet :: WWW/HTTP :: Dynamic Content",
        "Topic :: Software Development :: Libraries :: Python Modules",
        "Topic :: Internet :: WWW/HTTP :: Site Management",
        "Topic :: Software Development :: Documentation",
        "Topic :: Text Processing :: Filters",
        "Topic :: Text Processing :: Markup :: HTML",
        "Topic :: Text Processing :: Markup :: Markdown",
        "Topic :: Utilities",
    ],
    install_requires=[
        "markdown>=3.0",
        "pygments>=2.0", 
        "markdown-mermaidjs>=1.0",
    ],
    python_requires=">=3.7",
    entry_points={
        "console_scripts": [
            "mdtree=mdtree.main:main",
        ],
    },
)



#!/usr/bin/env python
# coding=utf-8
import os
import datetime
from mdtree.mdutils import to_unicode, utf8, clean_list

_d_static_path = os.path.join(os.path.dirname(os.path.abspath(__file__)), "static")


def adjust_ext_list(l1, l2):
    """
    remove extensions from l1 which were defined in l2
    :param list l1: source list
    :param list l2: define list
    :return list:
    """
    removed_ext_list = [i for i in l2 if i.startswith("-")]
    return [i for i in l1 if "-%s" % i not in removed_ext_list]


def gen_html(params):
    """
    Generate html
    :return str: html
    """
    with open(os.path.join(_d_static_path, "html/template.html")) as f:
        _tpl = f.read()
        _tpl = to_unicode(_tpl)

    title = to_unicode(params["title"])
    css_more = to_unicode(params["css_more"])
    js_more = to_unicode(params["js_more"])
    css_base = to_unicode(params["css_base"])
    js_base = to_unicode(params["js_base"])
    content = to_unicode(params["content"])
    toc = to_unicode(params["toc"])

    generated_at = """<div id="generated-at">
        Generated at: %s
    </div>
            """ % datetime.datetime.now().replace(microsecond=0).__str__()
    generated_at = to_unicode(generated_at)

    html = _tpl.format(title=title, content=content, css_base=css_base, js_base=js_base,
                       toc_content=toc, css_more=css_more, js_more=js_more,
                       generated_at=generated_at)

    return utf8(html)


def prepare_static_files():
    """
    prepare static files
    """
    __css_file = os.path.join(_d_static_path, "css/mdtree.min.css")
    __js_file = os.path.join(_d_static_path, "js/mdtree.min.js")
    with open(__css_file) as f:
        _css_base = f.read()
    with open(__js_file) as f:
        _js_base = f.read()
    return _css_base, _js_base


def parse_static_files(meta, css_list, js_list):
    """
    Get more static files
    :param dict meta: markdown meta
    :param list css_list: css list
    :param list js_list: js list
    :return tuple: css, js
    """
    css_more = list(set(css_list + clean_list(meta.get("css", []))))
    js_more = list(set(js_list + clean_list(meta.get("js", []))))

    css = ["""<link href="%s" rel="stylesheet" type="text/css" />""" % c for c in css_more]
    js = ["""<script type="text/javascript" src="%s"></script>""" % j for j in js_more]

    css_more_str = "\n".join(css)
    js_more_str = "\n".join(js)

    return css_more_str, js_more_str
